package com.shigongli.task;

import cn.binarywang.wx.miniapp.api.WxMaService;
import cn.binarywang.wx.miniapp.bean.WxMaSubscribeMessage;
import com.shigongli.common.WxUtils;
import com.shigongli.config.WeixinConfig;
import com.shigongli.entity.TaPerson;
import com.shigongli.service.ITaPersonService;
import lombok.extern.slf4j.Slf4j;
import me.chanjar.weixin.common.error.WxErrorException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;


/**
 * 发送离店提醒
 */
@Slf4j
@Component
public class SendCheckOutMessage {

    @Autowired
    ITaPersonService iTaPersonService;

    @Autowired
    WeixinConfig weixinConfig;

    @Autowired
    WxUtils wxUtils;

    @Scheduled(cron = "0 0 11 * * ?")
    public void run() {
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
        String today = LocalDateTime.now().format(formatter);

        List<TaPerson> taPersonList = iTaPersonService.getCheckoutListBy(today);
        if (null == taPersonList) {
            return;
        }

        // 找到消息模板
        List<WeixinConfig.Ma.SubscribeMessage> subscribeMessages = weixinConfig.getMa().getSubscribeMessages();
        WeixinConfig.Ma.SubscribeMessage subscribeMessage = null;
        if (null == subscribeMessages || subscribeMessages.size() < 1) {
            log.error("没有找到小程序消息提醒配置");
            return;
        }
        for (WeixinConfig.Ma.SubscribeMessage message : subscribeMessages) {
            if ("check-out".equals(message.getType())) {
                subscribeMessage = message;
                break;
            }
        }
        if (null == subscribeMessage) {
            log.error("没有找到退房提醒的消息配置");
            return;
        }

        String remark = "请记得带好您的随身物品";
        WxMaService maService = wxUtils.getMaService();

        // 每隔 1s 发送一条离店提醒
        for (TaPerson taPerson : taPersonList) {
            List<WxMaSubscribeMessage.Data> dataList = new ArrayList<>();
            dataList.add(new WxMaSubscribeMessage.Data("time1", String.format("%s 12:00", today)));
            dataList.add(new WxMaSubscribeMessage.Data("thing2", remark));

            WxMaSubscribeMessage wxMaSubscribeMessage = WxMaSubscribeMessage
                    .builder()
                    .templateId(subscribeMessage.getCode())
                    .toUser(taPerson.getOpenid())
                    .page(String.format("/pages/index/index?orderId=%s&houseId=%s", taPerson.getOrderId(), taPerson.getHouseId()))
                    .data(dataList)
                    .build();

            try {
                maService.getMsgService().sendSubscribeMsg(wxMaSubscribeMessage);
                Thread.sleep(1000);
            } catch (WxErrorException e) {
                log.error("发送退房消息失败: {}", e.getMessage());
                e.printStackTrace();
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
    }
}
